@@ -1,8 +1,8 @@
 /******************************************************************************
-    (c) 1995-1998 E.M. Serrat             emserrat@geocities.com
+    (c) 1995-2002 E.M. Serrat             emserrat@geocities.com
+
+    Conversion to dnetd (c) 1999 by Christine Caulfield
 
-    Conversion to dnetd (c) 1999 by Patrick Caulfield
-    
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -61,11 +61,11 @@ void cterm_setchar(void)
 					0x07,0x02,	/*output ESC seq*/
 							/* Recognition	*/
 					0x01,0x00,
+
 					0x02,0x02,0x19,
 					0xff,0x00
 				      };
 
-
 	if (write(net,cterm_setchar_msg,sizeof(cterm_setchar_msg)) <  0)
 	{
 		perror("Cterm setchar");
@@ -81,7 +81,7 @@ void cterm_bind(void)
 				    0x01,0x00,		/* init messg */
 				    0x01,0x04,0x00,	/* Version     */
 				    0,0,0,0,0,0,0,0,    /* descrip     */
-				    0x01,0x02,0x00,0x02, 
+				    0x01,0x02,0x00,0x02,
 				    0x02,0x02,0xf4,0x03,
 				    0x03,0x04,0xfe,0x7f,0x00,0x00};
 	char	buf[512];
@@ -119,7 +119,7 @@ void cterm_bind(void)
 /*-----------------------------------------------------------------------*/
 void cterm_unbind (void)
 {
-	char	cterm_unbind_msg[] = {0x02,0x03,0x00};
+	char	cterm_unbind_msg[3] = {0x02,0x03,0x00};
 	write(net,cterm_unbind_msg,3);
 }
 /*-----------------------------------------------------------------------*/
@@ -130,6 +130,7 @@ void cterm_reset(int s)
 	char	*p;
 
 	cterm_unbind();
+	sleep(1); /* ugh!. Wait for data-ack to arrive */
 	p=line+sizeof("/dev/")-1;
 
 	setutent();
@@ -137,7 +138,7 @@ void cterm_reset(int s)
 	entry.ut_line[strlen(p)]='\0';
 	lentry=getutline(&entry);
 	lentry->ut_type=DEAD_PROCESS;
-	
+
 	memset(lentry->ut_line,0,UT_LINESIZE);
 	memset(lentry->ut_user,0,UT_NAMESIZE);
 	lentry->ut_time=0;
@@ -179,7 +180,7 @@ void cterm_read (void)
 {
 	char	cterm_stread_msg[] = {	0x09,0x00,0x31,0x00,
 					0x02,
-					0x40,0x48,0x01,
+					0x40,0x4A,0x02,
 					0xa0,0x00,0x00,0x00,
 					0x00,0x00,0x00,0x00,
 					0x00,0x00,0x00,0x00,
@@ -202,7 +203,7 @@ void cterm_write (char *buf)
 	short	*blklen;
 	int	wrtlen;
 	int     l;
-	
+
 	memcpy(&lclbuf[0],cterm_write_msg,9);
 	blklen=(short *)&lclbuf[2];
 	l = 5 + strlen(buf);
@@ -217,7 +218,7 @@ void cterm_write (char *buf)
 		perror("cterm_write");
 		exit(-1);
 	}
-	
+
 }
 /*-----------------------------------------------------------------------*/
 void cterm (void)
@@ -235,13 +236,13 @@ void cterm (void)
 	siga.sa_mask  = ss;
 	siga.sa_flags = 0;
 	sigaction(SIGCHLD, &siga, NULL);
-	
+
 	signal(SIGTSTP, SIG_IGN);
 	signal(SIGTTOU, SIG_IGN);
 
-	cterm_write("CTERM Version 1.0.4\r\nDECnet for Linux\r\n\n");
+	cterm_write("CTERM Version 1.0.6\r\nDECnet for Linux\r\n\n");
 	cterm_read();
-	
+
 	for (;;)
 	{
 		FD_ZERO(&rdfs);
@@ -255,14 +256,14 @@ void cterm (void)
 			cnt=read(pty,buf,sizeof(buf)-1);
 			if (cnt <= 0) break;
 			buf[cnt]='\0';
-			cterm_write(buf);			    
+			cterm_write(buf);
 		   }
 
 		   if (FD_ISSET(net,&rdfs))
 		   {
 			cnt=read(net,buf,sizeof(buf));
 			if (cnt <= 0) break;
-			
+
 			switch (buf[4])
 			{
 		 	case 0x03:
@@ -274,7 +275,7 @@ void cterm (void)
 				write(pty,&buf[12],cnt-12);
 				cterm_read();
 				break;
-		 	case 0x04: 
+		 	case 0x04:
 				if ( (buf[6] == 0x19) || (buf[6] == 3) )
 				{
 					buf[6]=0x03;
@@ -295,7 +296,7 @@ void doit  (void)
 	struct	stat	stb;
 	char	ptyname[] = "/dev/ptyCP";
 	int	gotpty =0;
-	
+
 	cterm_bind();
 
 #ifdef DNETUSE_DEVPTS
@@ -313,11 +314,11 @@ void doit  (void)
 		line[strlen("/dev/pty")] = c;
 		line[strlen("/dev/ptyC")] = '0';
 		if (stat(line,&stb) < 0)
-			break; 
+			break;
 		for (i=0; i < 16; i++)
 		{
 			line[strlen("/dev/ptyC")]= "0123456789abcdef"[i];
-			if ( (pty=open(line,O_RDWR)) > 0)	
+			if ( (pty=open(line,O_RDWR)) > 0)
 			{
 				gotpty = 1;
 				break;
@@ -325,7 +326,7 @@ void doit  (void)
 		}
 		if (gotpty) break;
 	}
-	if (!gotpty) 
+	if (!gotpty)
 	{
 		cterm_write("No ptys available for connection");
 		exit(-1);
@@ -333,13 +334,13 @@ void doit  (void)
 
 
 	line[strlen("/dev/")] = 't';
-	if ( (t=open(line,O_RDWR)) < 0) 
+	if ( (t=open(line,O_RDWR)) < 0)
 	{
 		cterm_write("Error connecting to physical terminal");
 		exit(-1);
 	}
 #endif
-	if ( fchmod(t,0) ) 
+	if ( fchmod(t,0) )
 	{
 		cterm_write("Error setting terminal mode");
 		exit(-1);
@@ -350,13 +351,13 @@ void doit  (void)
 		cterm_write("Error forking");
 		exit(-1);
 	}
-	if (i) 
+	if (i)
 	{
 		cterm();
 	}
-	
+
 	setsid();
-	
+
 	close(pty); close(net);
 	if (t != 0) dup2 (t,0);
 	if (t != 1) dup2 (t,1);
@@ -364,6 +365,9 @@ void doit  (void)
 
 	if (t > 2) close(t);
 
+	ioctl(0, TIOCSCTTY, (char *)NULL);
+
+
 	putenv("TERM=vt100");
 	execlp("/bin/login", "login", (char *)0);
 	cterm_write("Error executing login command");
@@ -388,15 +392,15 @@ int main(int argc, char *argv[])
     int debug = 0;
     char log_char = 's';
     char opt;
-    
-    // Deal with command-line arguments.
+
+    /* Deal with command-line arguments.*/
     opterr = 0;
     optind = 0;
     while ((opt=getopt(argc,argv,"?vVdhl:")) != EOF)
     {
-	switch(opt) 
+	switch(opt)
 	{
-	case 'h': 
+	case 'h':
 	    usage(argv[0], stdout);
 	    exit(0);
 
@@ -431,9 +435,9 @@ int main(int argc, char *argv[])
 	}
     }
 
-    // Initialise logging
+    /* Initialise logging */
     init_daemon_logging("ctermd", log_char);
-    
+
     net = dnet_daemon(DNOBJECT_CTERM, NULL, verbosity, !debug);
 
     if (net > -1)
