@@ -1,6 +1,6 @@
 /******************************************************************************
-    (c) 1998-1999 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
-    
+    (c) 1998-2009 Christine Caulfield         christine.caulfield@googlemail.com
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -21,10 +21,14 @@
 #include "file.h"
 #include "unixfile.h"
 
-// basename() is in libc but not in my header files 
-extern "C" char *basename(const char *);
+#ifdef __FreeBSD__
+#include <libgen.h>
+#endif
 
-int unixfile::open(char *mode)
+// basename() is in libc but not in my header files
+//extern "C" char *basename(const char *);
+
+int unixfile::open(const char *mode)
 {
 
 // if the filename was "-" then open standard in/output
@@ -50,7 +54,7 @@ int unixfile::open(char *mode)
 
 // This open routine is called when the output file is a directory (ie a
 // copy from multiple sources) so we don't need to check for stdin/out.
-int unixfile::open(char *basename, char *mode)
+int unixfile::open(const char *basename, const char *mode)
 {
     strcpy(printname, filename);
     strcat(printname, "/");
@@ -68,7 +72,7 @@ int unixfile::open(char *basename, char *mode)
 int unixfile::read(char *buf, int len)
 {
     if (transfer_mode == MODE_RECORD)
-    {	
+    {
 	char *out = buf;
 	char *in  = record_buffer + record_ptr;
 	char *endin  = record_buffer+record_buflen;
@@ -81,7 +85,7 @@ int unixfile::read(char *buf, int len)
 	    {
 		record_buflen = ::fread(record_buffer, 1, RECORD_BUFSIZE, stream);
 		record_ptr = 0;
-		if (record_buflen <= 0) 
+		if (record_buflen <= 0)
 		{
 		    if (reclen)
 			return reclen;
@@ -121,7 +125,7 @@ int unixfile::read(char *buf, int len)
 	reclen = ::fread(buf, 1, len, stream);
 	if (reclen <= 0)
 	    return -1;
-	
+
         // For block-mode we pad out the block to the
         // full block size. We never return a partial block.
         if (reclen != (int)block_size)
@@ -141,7 +145,7 @@ int unixfile::write(char *buf, int len)
 bool unixfile::eof()
 {
 // Don't know why this is necessary
-#ifdef __NetBSD__
+#if defined(__NetBSD__) || defined(__FreeBSD__)
     return feof(stream);
 #else
     return ::feof(stream);
@@ -158,7 +162,7 @@ int unixfile::close()
     return status;
 }
 
-void unixfile::perror(char *msg)
+void unixfile::perror(const char *msg)
 {
     ::perror(msg);
 }
@@ -171,7 +175,7 @@ bool unixfile::isdirectory()
     return S_ISDIR(s.st_mode);
 }
 
-int unixfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode) 
+int unixfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode, int flags, int timeout)
 {
 // Save these for later
     user_rfm = rfm;
@@ -184,11 +188,11 @@ int unixfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode)
     {
 	record_buffer = (char *)malloc(RECORD_BUFSIZE);
     }
-    
+
     return 0;
 };
 
-int unixfile::next() 
+int unixfile::next()
 {
     return FALSE;
 };
@@ -205,7 +209,7 @@ char *unixfile::get_printname(char *filename)
 {
     static char realname[MAX_PATH];
     static char tmpname[MAX_PATH];
-    
+
     strcpy(tmpname, this->filename);
     strcat(tmpname, "/");
     strcat(tmpname, filename);
@@ -228,7 +232,7 @@ bool unixfile::iswildcard()
 
 // Return a string telling the user whether we transferred blocks or
 // records or what.
-char *unixfile::get_format_name()
+const char *unixfile::get_format_name()
 {
 
     switch (transfer_mode)
@@ -274,9 +278,9 @@ unixfile::~unixfile()
 	free(record_buffer);
 }
 
-unixfile::unixfile(char *name)
+unixfile::unixfile(const char *name)
 {
-    strcpy(filename, name); 
+    strcpy(filename, name);
     record_buffer = NULL;
     record_ptr = record_buflen = 0;
 }
