@@ -6,10 +6,12 @@
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
  *
- * Authors:     Patrick Caulfield <patrick@debian.org>
+ * Authors:     Christine Caulfield <christine.caulfield@googlemail.com>
  *
  */
 
+#include <sys/types.h>
+#include <linux/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -21,6 +23,7 @@
 #include <sys/time.h>
 #include <netinet/in.h>
 #include <string.h>
+#include <limits.h>
 #include <linux/netfilter_decnet.h>
 #include <netdnet/dnetdb.h>
 #include <features.h>    /* for the glibc version number */
