@@ -1,6 +1,6 @@
 /******************************************************************************
-    (c) 1998      P.J. Caulfield          patrick@tykepenguin.cix.co.uk
-                  K.   Humborg            kenn@avalon.wombat.ie
+    (c) 1998,2008   Christine Caulfield     christine.caulfield@googlemail.com
+                    K.   Humborg            kenn@avalon.wombat.ie
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ static  struct	nodeent		*np;
 static  char			node[20];
 static  int			sockfd;
 static  int                     timeout = 60;
+static  int                     connect_timeout = 60;
 
 struct	sockaddr_dn		sockaddr;
 struct	accessdata_dn		accessdata;
@@ -54,6 +55,7 @@ static  int                     binary_mode;
 
 static int parse(char *);
 static void usage(FILE *);
+static char *connerror(int sockfd);
 /*-------------------------------------------------------------------------*/
 
 /*
@@ -80,7 +82,11 @@ void be_interactive(void)
 	    len = read(sockfd, buf, sizeof(buf));
 	    if (len < 0)
 	    {
-	        perror("reading from VMS");
+		fprintf(stderr, "Read failed: %s\n", connerror(sockfd));
+		return;
+	    }
+	    if (len == 0) /* EOF */
+	    {
 		return;
 	    }
 
@@ -129,7 +135,10 @@ void print_output(void)
     {
         if (len == -1)
 	{
-	    if (errno != ENOTCONN)  perror("Error reading from network");
+	    if (errno != ENOTCONN)
+		perror("Error reading from network");
+	    else
+		fprintf(stderr, "Read failed: %s\n", connerror(sockfd));
 	    break;
         }
 
@@ -151,6 +160,8 @@ void print_output(void)
  */
 int setup_link(void)
 {
+    struct timeval timeout = {connect_timeout,0};
+
     if ( (np=getnodebyname(node)) == NULL)
     {
 	printf("Unknown node name %s\n",node);
@@ -176,6 +187,9 @@ int setup_link(void)
 	strcpy(filename, "TASK");
     }
 
+    if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &timeout, sizeof(timeout)))
+	    perror("Error setting snd timeout");
+
     // Provide access control and proxy information
     if (setsockopt(sockfd,DNPROTO_NSP,SO_CONACCESS,&accessdata,
 		   sizeof(accessdata)) < 0)
@@ -201,7 +215,7 @@ int setup_link(void)
 
     if (connect(sockfd, (struct sockaddr *)&sockaddr, sizeof(sockaddr)) < 0)
     {
-	perror("connect");
+	fprintf(stderr, "Connect failed: %s\n", connerror(sockfd));
 	exit(-1);
     }
     return TRUE;
@@ -226,7 +240,7 @@ int main(int argc, char *argv[])
 /* Get command-line options */
     opterr = 0;
     optind = 0;
-    while ((opt=getopt(argc,argv,"?hVibt:")) != EOF)
+    while ((opt=getopt(argc,argv,"?hVibt:T:")) != EOF)
     {
 	switch(opt)
 	{
@@ -246,6 +260,10 @@ int main(int argc, char *argv[])
 	    timeout = atoi(optarg);
 	    break;
 
+	case 'T':
+	    connect_timeout = atoi(optarg);
+	    break;
+
 	case 'V':
 	    printf("\ndntask from dntools version %s\n\n", VERSION);
 	    exit(1);
@@ -468,12 +486,76 @@ static void usage(FILE *f)
     fprintf(f, "\nOptions:\n");
     fprintf(f, "  -i           Interact with the command procedure\n");
     fprintf(f, "  -t           Timeout (in seconds) for interactive command procedure input\n");
+    fprintf(f, "  -T <secs>    Connect timeout (default 60 seconds)\n");
     fprintf(f, "  -b           Treat received data as binary data\n");
     fprintf(f, "  -? -h        display this help message\n");
     fprintf(f, "  -V           show version number\n");
 
     fprintf(f, "\nExamples:\n\n");
     fprintf(f, " dntask 'myvax::'  - defaults to TASK.COM and proxy username\n");
-    fprintf(f, " dntask -i 'clustr\"patrick thecats\"::do_dcl.com'\n");
+    fprintf(f, " dntask -i 'clustr\"christine thecats\"::do_dcl.com'\n");
     fprintf(f, "\n");
 }
+
+// Return the text of a connection error
+static char *connerror(int sockfd)
+{
+#ifdef DSO_CONDATA
+    struct optdata_dn optdata;
+    unsigned int len = sizeof(optdata);
+    char *msg;
+
+    if (errno == ETIMEDOUT || getsockopt(sockfd, DNPROTO_NSP, DSO_DISDATA,
+		   &optdata, &len) == -1)
+    {
+	return strerror(errno);
+    }
+
+    // Turn the rejection reason into text
+    switch (optdata.opt_status)
+    {
+    case DNSTAT_REJECTED: msg="Rejected by object";
+	break;
+    case DNSTAT_RESOURCES: msg="No resources available";
+	break;
+    case DNSTAT_NODENAME: msg="Unrecognised node name";
+	break;
+    case DNSTAT_LOCNODESHUT: msg="Local Node is shut down";
+	break;
+    case DNSTAT_OBJECT: msg="Unrecognised object";
+	break;
+    case DNSTAT_OBJNAMEFORMAT: msg="Invalid object name format";
+	break;
+    case DNSTAT_TOOBUSY: msg="Object too busy";
+	break;
+    case DNSTAT_NODENAMEFORMAT: msg="Invalid node name format";
+	break;
+    case DNSTAT_REMNODESHUT: msg="Remote Node is shut down";
+	break;
+    case DNSTAT_ACCCONTROL: msg="Login information invalid at remote node";
+	break;
+    case DNSTAT_NORESPONSE: msg="No response from object";
+	break;
+    case DNSTAT_NODEUNREACH: msg="Node Unreachable";
+	break;
+    case DNSTAT_MANAGEMENT: msg="Abort by management/third party";
+	break;
+    case DNSTAT_ABORTOBJECT: msg="Remote object aborted the link";
+	break;
+    case DNSTAT_NODERESOURCES: msg="Node does not have sufficient resources for a new link";
+	break;
+    case DNSTAT_OBJRESOURCES: msg="Object does not have sufficient resources for a new link";
+	break;
+    case DNSTAT_BADACCOUNT: msg="The Account field in unacceptable";
+	break;
+    case DNSTAT_TOOLONG: msg="A field in the access control message was too long";
+	break;
+    default: msg=strerror(errno);
+	break;
+    }
+    return msg;
+#else
+    return strerror(errno);
+
+#endif
+}
