@@ -1,7 +1,7 @@
 /******************************************************************************
     connection.cc from libdap
 
-    Copyright (C) 1998-1999 Patrick Caulfield       patrick@tykepenguin.cix.co.uk
+    Copyright (C) 1998-2009 Christine Caulfield       christine.caulfield@googlemail.com
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -90,6 +90,7 @@ void dap_connection::initialise(int verbosity)
     lasterror   = errstring;
     errstring[0]= '\0';
     closed      = false;
+    connect_timeout = 60;
 
 #ifdef NO_BLOCKING
     blocking_allowed = false; // More useful for debugging
@@ -118,6 +119,24 @@ void dap_connection::close()
     }
 }
 
+bool dap_connection::set_socket_buffer_size()
+{
+    // Make sure the kernel buffer is large enough for our blocks
+    if (setsockopt(sockfd, SOL_SOCKET, SO_SNDBUF, &blocksize, sizeof(blocksize)) < 0)
+    {
+        sprintf(errstring, "setsockopt (SNDBUF) failed: %s", strerror(errno));
+	lasterror = errstring;
+	return false;
+    }
+    if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, &blocksize, sizeof(blocksize)) < 0)
+    {
+        sprintf(errstring, "setsockopt (RCVBUF) failed: %s", strerror(errno));
+	lasterror = errstring;
+	return false;
+    }
+    return true;
+}
+
 // Create a DECnet socket
 void dap_connection::create_socket()
 {
@@ -252,7 +271,12 @@ bool dap_connection::do_connect(const char *node, const char *user,
     memcpy(s.sdn_add.a_addr, binadr->n_addr, sizeof(s.sdn_add.a_addr));
 
     // Try very hard to get the local username for proxy access
+#ifdef __FreeBSD__
+// FIXME: FreeBSD does not export prototype even if correct header is included
+    char *local_user = NULL;
+#else
     char *local_user = cuserid(NULL);
+#endif
     if (!local_user || local_user == (char *)0xffffffff)
 	local_user = getenv("LOGNAME");
 
@@ -273,11 +297,18 @@ bool dap_connection::do_connect(const char *node, const char *user,
     if (setsockopt(sockfd, DNPROTO_NSP, SO_CONACCESS, &accessdata,
 		   sizeof(accessdata)) < 0)
     {
-        sprintf(errstring, "setsockopt failed: %s", strerror(errno));
+        sprintf(errstring, "setsockopt (CONACCESS) failed: %s", strerror(errno));
 	lasterror = errstring;
 	return false;
     }
 
+    if (!set_socket_buffer_size())
+        return false;
+
+    // Set connect timeout
+    struct timeval timeout = {connect_timeout, 0};
+    setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &timeout, sizeof(timeout));
+
     if (::connect(sockfd, (struct sockaddr *)&s,
 		  sizeof(sockaddr)) < 0)
     {
@@ -342,7 +373,7 @@ int dap_connection::read(bool block)
     }
     if (buflen == 0)
     {
-	lasterror = "Remote end closed connection";
+	lasterror = (char *)"Remote end closed connection";
 	return false;
     }
 
@@ -538,6 +569,7 @@ int  dap_connection::check_length(int needed)
 void dap_connection::set_blocksize(int bs)
 {
     blocksize = bs;
+    set_socket_buffer_size();
 }
 
 int dap_connection::get_blocksize()
@@ -545,6 +577,12 @@ int dap_connection::get_blocksize()
     return blocksize;
 }
 
+// You (obviously) must call this before connecting
+void dap_connection::set_connect_timeout(int seconds)
+{
+    connect_timeout = seconds;
+}
+
 //
 // Wait for an incoming connection
 // Returns a constructed new dap_connection object
@@ -558,6 +596,8 @@ dap_connection *dap_connection::waitfor()
     // Set up the listing context
     if (!listening)
     {
+        set_socket_buffer_size();
+
 	status = listen(sockfd, 5);
 	if (status)
 	{
@@ -755,7 +795,7 @@ bool dap_connection::parse(const char *fname,
 		if (n1 >= MAX_NODE ||
 		    fname[n0] == ' ' || fname[n0] == '\n')
 		{
-		    lasterror = "File name parse error";
+		    lasterror = (char *)"File name parse error";
 		    return false;
 		}
 		node[n1++] = fname[n0++];
@@ -782,7 +822,7 @@ bool dap_connection::parse(const char *fname,
 	    {
 		if (n1 >= MAX_USER)
 		{
-		    lasterror = "File name parse error";
+		    lasterror = (char *)"File name parse error";
 		    return false;
 		}
 		accessdata.acc_user[n1++] = fname[n0++];
@@ -810,7 +850,7 @@ bool dap_connection::parse(const char *fname,
 	    {
 		if (n1 >= MAX_PASSWORD)
 		{
-		    lasterror = "File name parse error";
+		    lasterror = (char *)"File name parse error";
 		    return false;
 		}
 		accessdata.acc_pass[n1++] = fname[n0++];
@@ -838,7 +878,7 @@ bool dap_connection::parse(const char *fname,
 	    {
 		if (n1 >= MAX_ACCOUNT)
 		{
-		    lasterror = "File name parse error";
+		    lasterror = (char *)"File name parse error";
 		    return false;
 		}
 		accessdata.acc_acc[n1++] = fname[n0++];
@@ -867,7 +907,7 @@ bool dap_connection::parse(const char *fname,
     binadr = getnodebyname(node);
     if (!binadr)
     {
-	lasterror = "Unknown or invalid node name ";
+	lasterror = (char *)"Unknown or invalid node name ";
 	return false;
     }
 
@@ -897,12 +937,12 @@ int dap_connection::send_crc(unsigned short crc)
 // A couple of general utility methods:
 void dap_connection::makelower(char *s)
 {
-    for (unsigned int i=0; i<strlen(s); i++) s[i] = tolower(s[i]);
+    for (unsigned int i=0; s[i]; i++) s[i] = tolower(s[i]);
 }
 
 void dap_connection::makeupper(char *s)
 {
-    for (unsigned int i=0; i<strlen(s); i++) s[i] = toupper(s[i]);
+    for (unsigned int i=0; s[i]; i++) s[i] = toupper(s[i]);
 }
 
 // Always returns false. Sets the error string to strerror(errno)
@@ -960,12 +1000,12 @@ bool dap_connection::exchange_config()
 }
 
 // Return the text of a connection error
-char *dap_connection::connerror(char *default_msg)
+const char *dap_connection::connerror(char *default_msg)
 {
 #ifdef DSO_CONDATA
     struct optdata_dn optdata;
     unsigned int len = sizeof(optdata);
-    char *msg;
+    const char *msg;
 
     if (getsockopt(sockfd, DNPROTO_NSP, DSO_DISDATA,
 		   &optdata, &len) == -1)
