@@ -47,6 +47,7 @@ class dap_connection
     int  get_remote_os() { return remote_os; };
     bool exchange_config();
     void clear_output_buffer();
+    void set_connect_timeout(int seconds);
     
 // Static utility functions
     static void makelower(char *s);
@@ -70,6 +71,7 @@ class dap_connection
     int    last_msg_start;
     int    end_of_msg;
     int    remote_os;
+    int    connect_timeout;
     struct nodeent *binadr;
     
     char *lasterror;
@@ -79,17 +81,18 @@ class dap_connection
 
     void create_socket();
     void initialise(int);
+    bool set_socket_buffer_size();
     bool do_connect(const char *node, const char *user,
 		    const char *password, sockaddr_dn &sockaddr);
 
     bool error_return(char *);
-    char *connerror(char *);
+    const char *connerror(char *);
     
 /* DECnet phase IV limits */
     static const int MAX_NODE     = 6;
     static const int MAX_USER     = 12;
-    static const int MAX_PASSWORD = 12;
-    static const int MAX_ACCOUNT  = 12;
+    static const int MAX_PASSWORD = 40;
+    static const int MAX_ACCOUNT  = 40;
 
  public:
     // Some DECnet object numbers
