@@ -1,7 +1,7 @@
 /*
     protocol.cc from libdap
 
-    Copyright (C) 1998-2001 Patrick Caulfield       patrick@tykepenguin.cix.co.uk
+    Copyright (C) 1998-2001 Christine Caulfield       christine.caulfield@googlemail.com
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -510,13 +510,13 @@ unsigned char dap_message::get_type()
     return msg_type;
 }
 
-char *dap_message::type_name()
+const char *dap_message::type_name()
 {
     return type_name(msg_type);
 }
 
 // Return the message type by name
-char *dap_message::type_name(int msg_type)
+const char *dap_message::type_name(int msg_type)
 {
     static char name[32];
 
@@ -605,7 +605,7 @@ bool dap_config_message::write(dap_connection &c)
     filesys.set_byte(0,3); // RMS-32 (hee hee!)
     filesys.write(c);
     version.set_byte(0,(unsigned char)7);
-    version.set_byte(1,(unsigned char)1);
+    version.set_byte(1,(unsigned char)2);
     version.set_byte(2,(unsigned char)0);
     version.set_byte(3,(unsigned char)5);
     version.set_byte(4,(unsigned char)0);
@@ -1351,7 +1351,7 @@ int dap_status_message::errno_to_stscode()
     }
 }
 
-char *dap_status_message::get_message()
+const char *dap_status_message::get_message()
 {
 // MACCODEs
     switch (stscode.get_int() >> 12)
@@ -1829,7 +1829,7 @@ char *dap_status_message::get_message()
 	case 0320: return "index not initialized.";
 	case 0321: return "illegal file attributes (corrupt file header).";
 	case 0322: return "index bucket fill size > bucket size in XAB.";
-	case 0323: return "key name buffer not readable or writeable in XAB.";
+	case 0323: return "key name buffer not readable or writable in XAB.";
 	case 0324: return "index bucket will not hold two keys  for  key  of reference.";
 	case 0325: return "multi-buffer count invalid (negative value).";
 	case 0326: return "network operation failed at remote node.";
@@ -2121,8 +2121,9 @@ char *dap_date_message::make_y2k(char *dt)
     return y2kdate;
 }
 
-char *dap_date_message::months[]={"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
-				  "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
+const char *dap_date_message::months[]=
+				{"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
+				 "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
 
 //------------------------- dap_alloc_message() ------------------------------
 
@@ -2217,7 +2218,7 @@ mode_t dap_protect_message::get_mode()
     return mode;
 }
 
-char *dap_protect_message::get_protection()
+const char *dap_protect_message::get_protection()
 {
     static char protstring[60];
     int p = 0;
@@ -2257,7 +2258,7 @@ char *dap_protect_message::get_protection()
     return protstring;
 }
 
-char *dap_protect_message::get_owner()
+const char *dap_protect_message::get_owner()
 {
     return owner.get_string();
 }
