@@ -7,6 +7,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include "rms.h"
@@ -14,15 +15,18 @@
 int main(int argc, char *argv[])
 {
     /* Open the file. Keep the RMSHANDLE returned */
-    RMSHANDLE h = rms_t_open("marsha::index.dat", O_RDWR, NULL);
+    RMSHANDLE h = rms_t_open("ford::index.dat", O_RDWR, NULL);
     if (h)
     {
         char b[10240];
+	char key[256];
+	int keylen = 7;
 	int got;
 
 	/* Look for the record with my name in it. We don't need rac=key
 	   here because librms gives us it for free */
-	got = rms_t_read(h, b, sizeof(b), "key=PATRICK,kop=kge");
+	memcpy(key, "\0\0\0CHRISSIE\0", keylen);
+	got = rms_t_read(h, b, sizeof(b), "ksz=%d,key=%*s,kop=kge", keylen, keylen, key);
 	if (got > 0)
 	{
 	    b[got] = '\0';
@@ -39,7 +43,7 @@ int main(int argc, char *argv[])
 	{
 	    fprintf(stderr, "Update failed: %s\n", rms_lasterror(h));
 	}
-	
+
 	rms_t_close(h);
     }
     else
